#!/usr/bin/env python

import argparse
import os
import sys

try:
    import lz4tools
except ImportError:
    import __init__ as lz4tools

parser=argparse.ArgumentParser()

parser.add_argument('-f', action='store_true', dest='file',   default=False,
                    help='Compress file. Default action if input is a file.')
parser.add_argument('-t', action='store_true', dest='tar',    default=False,
                    help=''.join(['Compress directory to .tar.lz4. ',
                                  'Default action if input is a directory']))
parser.add_argument('-d', action='store_true', dest='decomp', default=False,
                    help=''.join(['Decompress file. Default action if the file ',
                                  'ends in .lz4.']))
parser.add_argument('-i', action='store_true', dest='info', default=False,
                    help=''.join(['Print frame information from the file\'s ',
                                  'header.']))
parser.add_argument('-bs', action='store', dest='blkSizeId', default=7, type=int,
                    help=''.join(['Specify blkSizeId. Valid values are 4-7. ',
                                  'Default value is 7.']), choices=range(4, 8)),
parser.add_argument('-bm', action='store', dest='blkMode', default=0, type=int,
                    help=''.join(['Specify blkMode. 0 = Chained blocks. ',
                                  '1 = Independent blocks Default value is 0.']),
                    choices=[0, 1])
parser.add_argument('input', action='store', help='The targeted input.')
parser.add_argument('output', action='store', nargs='?', default=None,
                    help='Optional output target.')
#parser.add_argument('-dt', action='store_true', dest='decompTar',
#                  help=''.join(['Decompress tar. Default action if the file',
#                                ' ends in .tar.lz4.']))
if len(sys.argv) == 1:
    parser.print_help()
    sys.exit()
res = parser.parse_args()

#print('blockSizeId: {}'.format(res.blkSizeId))
prefs = lz4tools.lz4f.makePrefs(res.blkSizeId, res.blkMode)

compFile = lambda: lz4tools.compressFileDefault(res.input, outname=res.output, prefs=prefs)
compDir = lambda: lz4tools.compressTarDefault(res.input, outname=res.output, prefs=prefs)
decompFile = lambda: lz4tools.decompressFileDefault(res.input, outname=res.output)
getInfo = lambda: lz4tools.getFileInfo(res.input)
outErr = lambda: sys.stdout.write('Please specify only ony of the comp/decomp options')

if res.info:
    print(getInfo())
elif res.file:
    if res.tar or res.decomp:
        outErr()
    compFile()
elif res.tar:
    if res.decomp:
        outErr()
    compDir()
elif res.input.endswith('lz4'):
    decompFile()
elif os.path.isfile(res.input):
    compFile()
elif os.path.isdir(res.input):
    compDir()
else:
    parser.print_help()



import lz4f
import os
import sys

if sys.version_info.major >= 3:
    import builtins as __builtin__
    from .lz4file import Lz4File
    from .lz4tar import Lz4Tar
    from io import BytesIO as StringIO
else:
    import __builtin__
    from lz4file import Lz4File
    from lz4tar import Lz4Tar
    from StringIO import StringIO


__all__ = ['lz4file', 'lz4tar']

def compressFileDefault(name, overwrite=False, outname=None, prefs=None):
    """
    :type string: name      - name of file to compress
    :type bool:   overwrite - overwrite destination
    :type string: outname   - name for compressed file, not required.
                              Default will be '.'.join([name, 'lz4'])
    Generic compress method for a file. Adds .lz4 to original file name for
    output, unless outname is provided.

    ***NOTE*** No longer uses compressFrame. This is now large file safe!
    It will now read the input in 64Kb chunks.
    """
    if not outname:
        outname = '.'.join([name, 'lz4'])
    if os.path.exists(outname):
        print('File Exists!')
        if not overwrite:
            return
    if not os.path.exists(name):
        print('Unable to locate the original file. Please check filename.')
        return
    cCtx = lz4f.createCompContext()
    header = lz4f.compressBegin(cCtx, prefs)
    with __builtin__.open(outname, 'wb') as out:
        out.write(header)
        with __builtin__.open(name, 'rb') as infile:
            while True:
                decompData = infile.read((64*(1<<10)))
                if not decompData:
                    break
                compData = lz4f.compressUpdate(decompData, cCtx)
                out.write(compData)
            out.write(lz4f.compressEnd(cCtx))
        out.flush()
        out.close()
    lz4f.freeCompContext(cCtx)
def compressTarDefault(dirName, overwrite=None, outname=None, prefs=None):
    """
    :type string: dirName   - the name of the dir to tar
    :type bool:   overwrite - overwrite destination
    Generic compress method for creating .tar.lz4 from a dir.

    ***WARNING*** Currently uses StringIO object until lz4file supports write.
    Avoid using for large directories, it will consume quite a bit of RAM.
    """
    if not outname:
        outname = '.'.join([dirName.rstrip('/'), 'tar', 'lz4'])
    if not os.path.exists(dirName):
        print('Unable to locate the directory to compress.')
        return
    buff = StringIO()
    tarbuff = Lz4Tar.open(fileobj=buff, mode='w')
    tarbuff.add(dirName)
    tarbuff.close()
    buff.seek(0)
    cCtx = lz4f.createCompContext()
    header = lz4f.compressBegin(cCtx, prefs)
    with __builtin__.open(outname, 'wb') as out:
        out.write(header)
        while True:
            decompData = buff.read((64*(1<<10)))
            if not decompData:
                break
            compData = lz4f.compressUpdate(decompData, cCtx)
            out.write(compData)
        out.write(lz4f.compressEnd(cCtx))
        out.flush()
    lz4f.freeCompContext(cCtx)
    del tarbuff, buff
def decompressFileDefault(name, overwrite=False, outname=None):
    """
    :type string: name      - name of file to decompress
    :type bool:   overwrite - overwrite destination
    :type string: outname   - name for decompressed file, not required.
                              Default will be '.'.join([name, 'lz4'])
    Generic decompress method for a file. Removes .lz4 to original file name for
    output, unless outname is provided.

    ***WARNING*** Currently uses lz4f.compressFrame, which will read the entire
    original file into memory, then pass to c-module for compression. Avoid
    using this for large files until migrated to advCompress functions.
    """
    if not outname:
        outname = name.strip('.lz4')
        if outname == name:
            print(''.join(['File does not contain .lz4 extension. ',
                           'Please provide outname.']))
            return
        if os.path.exists(outname) and not overwrite:
            print(''.join(['Output file exists! Please authorize overwrite or',
                           ' specify a different outfile name.']))
    infile = Lz4File.open(name)
    infile.decompress(outname)
def getFileInfo(name):
    """
    :type string: name - name of file to examine
    Returns a dict object containing the file's header information.
    """
    if not os.path.exists(name):
        print('Unable to locate the file')
        return
    dCtx = lz4f.createDecompContext()
    with __builtin__.open(name, 'rb') as inFile:
        header = inFile.read(7)
    return lz4f.getFrameInfo(header, dCtx)
def open(name=None, fileObj=None):
    """  Alias for Lz4File.open()    """
    return Lz4File.open(name, fileObj)
def openTar(name=None, fileObj=None):
    """  Alias for Lz4Tar.open()     """
    return Lz4Tar.lz4open(name, 'r', fileObj)



import sys
import tarfile
if sys.version_info.major >= 3:
    import builtins as __builtin__
    from .lz4file import Lz4File
else:
    import __builtin__
    from lz4file import Lz4File

class Lz4Tar(tarfile.TarFile):
    @classmethod
    def lz4open(cls, name=None, mode='r', fileobj=None):
        if name and not fileobj:
            try:
                fileobj = Lz4File.open(name)
                #fileobj=__builtin__.open(name, 'rb')
            except IOError:
                raise ReadError('Not a lz4 file')
        elif not name and not fileobj:
            print('Unable to open without a name or fileobj')
            return
        if not name and hasattr(fileobj.name):
            name = fileobj.name
        try:
            t = cls(None, mode, fileobj)
        except:
            fileobj.close()
        t._extfileobj = False
        return t
    #tarfile.TarFile.OPEN_METH.update({'lz4': 'lz4open'})
    #tarfile.TarFile.lz4open = lz4open
    OPEN_METH = {
        "tar": "taropen",   # uncompressed tar
        "gz":  "gzopen",    # gzip compressed tar
        "bz2": "bz2open",   # bzip2 compressed tar
        "lz4": "lz4open"    # lz4 compressed tar
        }


#!/usr/bin/env python


from setuptools import setup, Extension

VERSION = (1, 2, 1, 7)
VERSION_STR = ".".join([str(x) for x in VERSION])

setup(
    name='lz4tools',
    version=VERSION_STR,
    description="LZ4Frame Bindings and tools for Python",
    license = 'BSD',
    long_description=open('README.rst', 'r').read(),
    author='Christopher Jackson',
    author_email='darkdragn.cj@gmail.com',
    url='https://github.com/darkdragn/lz4file',
    packages=['lz4tools'],
    scripts=['lz4toolsCli'],
    ext_modules=[
        Extension('lz4f', [
            'src/lz4.c',
            'src/lz4hc.c',
            'src/lz4frame.c',
            'src/python-lz4f.c',
            'src/xxhash.c'
        ], extra_compile_args=[
            "-std=c99",
            "-O3",
            "-Wall",
            "-W",
            "-Wundef",
            "-DVERSION=\"%s\"" % VERSION_STR,
            "-DLZ4_VERSION=\"r124\"",
        ])],
    setup_requires=["nose>=1.0"],
    test_suite = "nose.collector",
    keywords = ['lz4', 'lz4frame', 'lz4file', 'lz4tar'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



#!/usr/bin/python

import binascii
import lz4f
import os
import struct
import sys
import tarfile

if sys.version_info.major >= 3:
    import builtins as __builtin__
else:
    import __builtin__


class Lz4File:
    def __init__(self, name, fileObj=None, seekable=True):
        parseMagic = lambda x: binascii.hexlify(x[:4])
        self.name = name
        if fileObj:
            self.fileObj = fileObj
            self.compEnd = self.tell_end()
        else:
            return open(name)
        self.header = fileObj.read(7)
        if parseMagic(self.header) == b'04224d18':
            self.dCtx = lz4f.createDecompContext()
            self.fileInfo = lz4f.getFrameInfo(self.header, self.dCtx)
            self.blkSizeID = self.fileInfo.get('blkSize')
        else:
            raise IOError
        if seekable:
            try:
                self.load_blocks()
            except:
                #print('Unable to load blockDict. Possibly not a lz4 file.')
                raise IOError
    @classmethod
    def open(cls, name = None, fileObj = None, seekable=True):
        if not name and not fileObj:
            sys.stderr.write('Nothing to open!')
        if not fileObj:
            fileObj = __builtin__.open(name, 'rb')
        return cls(name, fileObj, seekable)
    def close(self):
        self.fileObj.close()
    def decompress(self, outName):
        """
        :type string: outName
        Generic decompress function. Will decompress the entire file to outName.
        """
        writeOut = __builtin__.open(outName, 'wb')
        for blk in self.blkDict.values():
            out = self.read_block(blk=blk)
            writeOut.write(out)
            writeOut.flush()
        writeOut.close()
    def get_block_size(self):
        """
        :type file: fileObj - At least seeked to 7 (past header)
        Static method to determine next block's blockSize.
        """
        size = struct.unpack('<I', self.fileObj.read(4))[0]
        self.fileObj.seek(self.fileObj.tell()-4)
        returnSize = (size & 0x7FFFFFFF)
        if not returnSize:
            return 0
        return returnSize+4
    def load_blocks(self):
        """
        Generates the blkDict, making the file seekable. Only one block will
        be decoded at a time, minimizing memory usage.
        """
        self.blkDict = {}
        total, blkNum, pos = 0, 0, 7
        blkSize = self.get_block_size()
        while blkSize > 0:
            data = self.read_block(blkSize, setCur=False)
            total += len(data)
            self.blkDict.update({blkNum: {'compressed_begin': pos,
                                'decomp_e': total, 'blkSize': blkSize}})
            blkNum += 1
            if not self.fileObj.tell() == self.compEnd:
                blkSize = self.get_block_size()
            else:
                break
            pos = self.fileObj.tell()
        self.end = total-1
        del data, total
        self.curBlk = 0
        self.decomp = self.read_block(blk=self.blkDict.get(0))
        self.seek(0)
    def read(self, size = None):
        """
        :type int: size
        File read-like function. If passed a size, it only reads those bytes.
        If not passed a size, it reads the entire file, from the current position.
        """
        out = bytes()
        decompOld = self.decompPos
        if size == 0:
            return ''
        if self.pos == self.end:
            raise EOFError("Reached EOF")
        if not size or self.pos+size > self.end:
            size = self.end-self.pos
        newPos = self.pos+size
        if self.decompPos+size > -1:
            out += self.decomp[decompOld:]
            size = size - len(out)
            self.pos = self.curBlkData.get('decomp_e')
            self.curBlk +=1
            self.decomp = self.read_block(blk=self.curBlkData, setCur=False)
            out += self.read(size)
        else:
            out += self.decomp[decompOld:decompOld+size]
            self.pos = newPos
            return out
        return out
    def read_block(self, blkSize = None, blk = None, setCur = True):
        """
        :type int:  blkSize - returned from get_block_size()
        :type dict: blk     - entry from blkDict
        :type bool: setCur  - update current blk var

        Reads the next block, unless provided a blk from blkDict. If provided
        a blk, it will read that specific block.
        """
        if blk:
            self.fileObj.seek(blk.get('compressed_begin'))
            blkSize = blk.get('blkSize')
        if not blkSize: blkSize = self.get_block_size()
        if blkSize == 0: return ''
        if setCur:
            try:
                iteritems = self.blkDict.iteritems
            except AttributeError:
                iteritems = self.blkDict.items
            self.curBlk = [num for num, b in iteritems()
                           if self.fileObj.tell() == b.get('compressed_begin')][0]
        if (self.fileObj.tell() + blkSize + 8) == self.compEnd:
            blkSize += 8
            regen = True
        compData = self.fileObj.read(blkSize)
        resultDict = lz4f.decompressFrame(compData, self.dCtx, self.blkSizeID)
        if 'regen' in locals(): self._regenDCTX()
        return resultDict.get('decomp')
    def seek(self, offset, whence=0):
        """
        :type int: offset
        File seek-like function. Accepts offset. Whence for future improvement,
        but not yet implemented.
        """
        thisBlk = int()
        if not offset:
            blk = self.blkDict.get(0)
        else:
            try:
                iteritems = self.blkDict.iteritems
            except AttributeError:
                iteritems = self.blkDict.items
            thisBlk, blk = [[num, b] for num, b in iteritems()
                   if offset < b.get('decomp_e')][0]
        if self.curBlk == thisBlk:
            self.pos = offset
        else:
            self.curBlk = thisBlk
            self.decomp =  self.read_block(blk = blk, setCur=False)
            self.pos = offset
    def tell(self):
        """
        Returns the current position in the 'decompressed' data.
        """
        return self.pos
    def tell_end(self):
        """
        :type file: fileObj
        Determine the end of the compressed file.
        """
        pos = self.fileObj.tell()
        self.fileObj.seek(0, 2)
        end = self.fileObj.tell()
        self.fileObj.seek(pos)
        return end
    def _regenDCTX(self):
        """
        Regenerate the decompression context.
        """
        try:
            lz4f.freeDecompContext(self.dCtx)
            del self.dCtx
            self.dCtx = lz4f.createDecompContext()
            frameInfo = lz4f.getFrameInfo(self.header, self.dCtx)
            lz4f.disableChecksum(self.dCtx)
        except AttributeError:
            self.dCtx = lz4f.createDecompContext()
            frameInfo = lz4f.getFrameInfo(self.header, self.dCtx)
            pass
        del frameInfo
    @property
    def decompPos(self):
        return self.pos - self.curBlkData.get('decomp_e')
    @property
    def curBlkData(self):
        return self.blkDict.get(self.curBlk)
    #@property
    def seekable(self):
        if self.blkDict:
            return True
        return False

